﻿using AspectInjector.Broker;
using Autodesk.Revit.UI;
using System;
using System.Linq;

namespace uBIM.Checker.AOP
{
#if !DEBUG
    [Injection(typeof(CheckerAspect))]
#endif
    public class CheckerAttribute : Attribute
    {
        public string Title { get; }
        public CheckerAttribute(string title = "uBIM")
        {
            Title = title;
        }

    }

    [Aspect(Scope.PerInstance)]
    public class CheckerAspect
    {
        [Advice(Kind.Around, Targets = Target.Method)]
        public object DoCheck([Argument(Source.Instance)] object obj, [Argument(Source.Triggers)] Attribute[] triggers, [Argument(Source.Target)] Func<object[], object> target, [Argument(Source.Arguments)] object[] args)
        {
            var checker = triggers.OfType<CheckerAttribute>().First();

            if (checker != null && obj is IExternalCommand)
            {
                if (LicenseChecker.Instance.CheckLicenseState(checker.Title))
                {
                    return target(args);
                }
                AboutForm aboutForm = new AboutForm(checker.Title);
                aboutForm.ShowDialog();
                return Result.Cancelled;
            }
            else
            {
                if (LicenseChecker.Instance.CheckLicenseState(checker.Title, false))
                {
                    return target(args);
                }
                AboutForm aboutForm = new AboutForm(checker.Title);
                aboutForm.ShowDialog();
                if (LicenseChecker.Instance.CheckLicenseState(checker.Title, false))
                {
                    return target(args);
                }
                throw new MethodAccessException($"{checker.Title}：激活信息无效");
            }
        }
    }
}