﻿using Microsoft.Win32;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Management;
using System.Net;
using System.Reflection;
using System.Runtime.ExceptionServices;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Security;
using System.Security.AccessControl;
using System.Security.Cryptography;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Xml;
using MessageBox = System.Windows.MessageBox;

namespace uBIM.Checker.AOP
{
    public class Common
    {
        /// <summary>
        /// 强制窗口在前
        /// </summary>
        /// <param name="hWnd"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool SetForegroundWindow(IntPtr hWnd);

        /// <summary>
        /// 激活窗口
        /// </summary>
        /// <param name="hWnd"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool SetActiveWindow(IntPtr hWnd);

        #region 试用信息文件路径
        public static readonly string TRY_INFO_FILE_PATH = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\" + AssemblyFileProductName() + "\\try_info.dat";
        #endregion

        #region 有效期的文件路径
        /// <summary>
        /// 有效期的文件路径
        /// </summary>
        public static readonly string LIMIT_DATE_FILE_PATH = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\" + AssemblyFileProductName() + "\\aga_limit.dat";
        #endregion

        #region 记录日期的文件路径
        /// <summary>
        /// 记录日期的文件路径
        /// </summary>
        public static readonly string RECORD_DATE_FILE_PATH = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\" + AssemblyFileProductName() + "\\aga_record.dat";
        #endregion

        #region 获取当前程序版本号
        /// <summary>
        /// 获取当前程序版本号
        /// </summary>
        /// <returns>版本号</returns>
        public static string AssemblyFileVersion()
        {
            object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyFileVersionAttribute), false);
            if (attributes.Length == 0)
            {
                return string.Empty;
            }
            else
            {
                return ((AssemblyFileVersionAttribute)attributes[0]).Version;
            }
        }
        #endregion

        #region 获取程序集产品名称
        /// <summary>
        /// 获取程序集产品名称
        /// </summary>
        /// <returns>产品名称</returns>
        public static string AssemblyFileProductName()
        {
            object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyProductAttribute), false);
            if (attributes.Length == 0)
            {
                return string.Empty;
            }
            else
            {
                return ((AssemblyProductAttribute)attributes[0]).Product;
            }
        }
        #endregion

        #region 获取设备识别码        
        /// <summary>
        /// 获取设备识别码
        /// </summary>
        /// <returns></returns>
        public static string GetDeviceCode(bool includeAssemblyFileProductName = true)
        {
            string cpuId = GetCPUID();
            string diskId = GetHardDiskID();
            if (includeAssemblyFileProductName)
            {
                return GetMd5Hash(AssemblyFileProductName() + string.Format("[{0}{1}{0}]", cpuId, diskId)).ToUpper();
            }
            else
            {
                return GetMd5Hash(string.Format("[{0}{1}{0}]", cpuId, diskId)).ToUpper();
            }
        }
        #endregion

        #region 获取CPU ID
        /// <summary>
        /// 获取CPU ID
        /// </summary>
        /// <returns></returns>
        public static string GetCPUID()
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher("SELECT ProcessorId FROM Win32_Processor WHERE (ProcessorId IS NOT NULL)"); //SELECT * FROM Win32_Processor
            string strCPUID = string.Empty;
            ManagementObjectCollection managementObjectCollection = searcher.Get();
            foreach (ManagementObject mo in managementObjectCollection)
            {
                strCPUID = (string)mo["ProcessorId"];
                break;
            }
            return strCPUID;
        }
        #endregion

        #region 获取第一块磁盘编号
        /// <summary>
        /// 获取第一块磁盘编号
        /// </summary>
        /// <returns></returns>
        public static string GetHardDiskID()
        {
            try
            {
                ManagementObjectSearcher searcher = new ManagementObjectSearcher("SELECT * FROM Win32_PhysicalMedia");
                string strHardDiskID = null;
                foreach (ManagementObject mo in searcher.Get())
                {
                    strHardDiskID = mo["SerialNumber"].ToString().Trim();
                    break;
                }
                return strHardDiskID;
            }
            catch
            {
                return string.Empty;
            }
        }
        #endregion

        #region MD5
        /// <summary>
        /// 取得输入字符串的MD5哈希值
        /// </summary>
        /// <param name="argInput">输入字符串</param>
        /// <returns>MD5哈希值</returns>
        public static string GetMd5Hash(string argInput)
        {
            // Create a new instance of the MD5CryptoServiceProvider object.
            MD5 md5Hasher = MD5.Create();

            // Convert the input string to a byte array and compute the hash.
            byte[] data = md5Hasher.ComputeHash(Encoding.Default.GetBytes(argInput));

            // Create a new Stringbuilder to collect the bytes
            // and create a string.
            StringBuilder sBuilder = new StringBuilder();

            // Loop through each byte of the hashed data 
            // and format each one as a hexadecimal string.
            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }

            // Return the hexadecimal string.
            return sBuilder.ToString();
        }

        /// <summary>
        /// 验证MD5哈希值
        /// </summary>
        /// <param name="argInput">输入字符串</param>
        /// <param name="argHash">哈希值</param>
        /// <returns>相同返回TRUE,不同返回FALSE</returns>
        public static bool VerifyMd5Hash(string argInput, string argHash)
        {
            // Hash the input.
            string hashOfInput = GetMd5Hash(argInput);

            // Create a StringComparer an comare the hashes.
            StringComparer comparer = StringComparer.OrdinalIgnoreCase;

            if (0 == comparer.Compare(hashOfInput, argHash))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion

        #region 写入字符串数据到文件
        /// <summary>
        /// 写入字符串数据到文件
        /// </summary>
        /// <param name="strData">字符串</param>
        /// <param name="strPath">文件</param>
        public static bool WirteStringData(string strData, string strPath)
        {
            bool result = false;
            try
            {
                string strLog = string.Empty;
                string saveFolder = Path.GetDirectoryName(strPath);
                if (!Directory.Exists(saveFolder))
                {
                    Directory.CreateDirectory(saveFolder);
                }
                //DirectorySecurity secu = new DirectorySecurity(Path.GetDirectoryName(strPath), AccessControlSections.Access);
                //FileSystemAccessRule fsar = new FileSystemAccessRule(Environment.UserName, FileSystemRights.FullControl, AccessControlType.Allow);
                //secu.SetAccessRule(fsar);
                //Directory.SetAccessControl(saveFolder, secu);
                using (StreamWriter sw = new StreamWriter(strPath, false))
                {
                    sw.WriteLine(strData);
                }
                result = true;
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return result;
        }

        /// <summary>
        /// 写入字符串数据到文件
        /// </summary>
        /// <param name="strData">字符串</param>
        /// <param name="strPath">文件</param>
        /// <param name="msg">异常消息</param>
        /// <returns></returns>
        public static bool WirteStringData(string strData, string strPath, ref string msg)
        {
            bool result = false;
            try
            {
                WirteStringData(strData, strPath);
                result = true;
            }
            catch (Exception ex)
            {
                msg = string.Format("{0}\r\n\r\n{1}", ex.uBIMMessage(), ex.uBIMMessage());
            }
            return result;
        }
        #endregion

        #region 从文件读取字符串
        /// <summary>
        /// 从文件读取字符串
        /// </summary>
        /// <param name="strPath">文件路径</param>
        /// <returns></returns>
        public static string ReadStringData(string strPath)
        {
            string result = string.Empty;
            try
            {
                using (StreamReader sr = new StreamReader(strPath))
                {
                    result = sr.ReadToEnd();
                }
            }
            catch
            {
                throw;
            }
            return result;
        }
        #endregion

        #region 对象转JSON
        /// <summary>
        /// 对象转JSON
        /// </summary>
        /// <typeparam name="T">类型约束</typeparam>
        /// <param name="t">类型</param>
        /// <returns>返回JSON字符串</returns>
        public static string EntityToJsonString<T>(T t)
        {
            return JsonConvert.SerializeObject(t);
        }

        /// <summary>
        /// 对象转JSON
        /// </summary>
        /// <typeparam name="T">类型约束</typeparam>
        /// <param name="t">类型</param>
        /// <param name="settings">JsonSerializerSettings</param>
        /// <returns>返回JSON字符串</returns>
        public static string EntityToJsonString<T>(T t, JsonSerializerSettings settings)
        {
            return JsonConvert.SerializeObject(t, settings);
        }
        #endregion

        #region JSON转对象
        /// <summary>
        /// JSON转对象
        /// </summary>
        /// <typeparam name="T">类型约束</typeparam>
        /// <param name="strJson">JSON字符串</param>
        /// <param name="t">类型</param>
        /// <returns>返回指定类型</returns>
        public static T JsonStringToEntity<T>(string strJson)// where T : class
        {
            try
            {
                return JsonConvert.DeserializeObject<T>(strJson);
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// JSON转对象
        /// </summary>
        /// <typeparam name="T">类型约束</typeparam>
        /// <param name="strJson">JSON字符串</param>
        /// <param name="t">类型</param>
        /// <param name="settings">JsonSerializerSettings</param>
        /// <returns>返回指定类型</returns>
        public static T JsonStringToEntity<T>(string strJson, JsonSerializerSettings settings)
        {
            try
            {
                return JsonConvert.DeserializeObject<T>(strJson, settings);
            }
            catch
            {
                throw;
            }
        }


        public static dynamic JsonStringToEntity<T>(string strJson, ExpandoObjectConverter expConverter)
        {
            try
            {
                return JsonConvert.DeserializeObject<T>(strJson, expConverter);
            }
            catch
            {
                throw;
            }
        }
        #endregion

        #region 将对象序列化
        /// <summary>
        /// 将对象序列化
        /// </summary>
        /// <param name="FilePath">文件(支持绝大多数数据类型)</param>
        /// <param name="obj">要序列化的对象(如哈希表,数组等等)</param>
        public static bool FileSerialize(string FilePath, object obj, ref string message)
        {
            bool result = false;
            FileStream fs = null;
            try
            {
                string saveFolder = Path.GetDirectoryName(FilePath);
                if (!Directory.Exists(saveFolder))
                {
                    Directory.CreateDirectory(saveFolder);
                }
                //DirectorySecurity secu = new DirectorySecurity(saveFolder, AccessControlSections.Access);
                //FileSystemAccessRule fsar = new FileSystemAccessRule(Environment.UserName, FileSystemRights.FullControl, AccessControlType.Allow);
                //secu.SetAccessRule(fsar);
                //System.IO.Directory.SetAccessControl(saveFolder, secu);
                fs = new FileStream(FilePath, FileMode.Create);
                BinaryFormatter sl = new BinaryFormatter();
                //sl.AssemblyFormat = System.Runtime.Serialization.Formatters.FormatterAssemblyStyle.Simple;
                sl.Binder = new Binder();
                sl.Serialize(fs, obj);
                result = true;
            }
            catch (Exception ex)
            {
                message = string.Format("{0}\r\n\r\n{1}", ex.uBIMMessage(), ex.uBIMMessage());
            }
            finally
            {
                if (fs != null)
                {
                    fs.Close();
                    fs.Dispose();
                }
            }
            return result;
        }
        #endregion

        #region 将文件反序列化
        /// <summary>
        /// 将文件反序列化
        /// </summary>
        /// <param name="FilePath">文件路径(必须是经过当前序列化后的文件)</param>
        /// <returns>返回 null 表示序列反解失败或者目标文件不存在</returns>
        public static object FileDeSerialize(string FilePath, ref string message)
        {
            object result = null;
            if (File.Exists(FilePath))
            {
                FileStream fs = null;
                try
                {
                    string saveFolder = Path.GetDirectoryName(FilePath);
                    if (!Directory.Exists(saveFolder))
                    {
                        Directory.CreateDirectory(saveFolder);
                    }
                    //DirectorySecurity secu = new DirectorySecurity(saveFolder, AccessControlSections.Access);
                    //FileSystemAccessRule fsar = new FileSystemAccessRule(Environment.UserName, FileSystemRights.FullControl, AccessControlType.Allow);
                    //secu.SetAccessRule(fsar);
                    //System.IO.Directory.SetAccessControl(saveFolder, secu);

                    fs = new FileStream(FilePath, FileMode.Open, FileAccess.Read, FileShare.Read);

                    BinaryFormatter sl = new BinaryFormatter();
                    //sl.AssemblyFormat = System.Runtime.Serialization.Formatters.FormatterAssemblyStyle.Simple;
                    sl.Binder = new Binder();
                    result = sl.Deserialize(fs);
                }
                catch (Exception ex)
                {
                    result = null;
                    message = string.Format("{0}\r\n\r\n{1}", ex.uBIMMessage(), ex.uBIMMessage());
                }
                finally
                {
                    if (fs != null)
                    {
                        fs.Close();
                    }
                }
            }
            return result;
        }
        #endregion

        #region 反序列化Byte数组
        /// <summary>
        /// 反序列化Byte数组
        /// </summary>
        /// <param name="resource"></param>
        /// <returns></returns>
        public static object DeSerializeByByteArray(byte[] resource)
        {
            try
            {
                using (MemoryStream sm = new MemoryStream(resource))
                {
                    BinaryFormatter sl = new BinaryFormatter();
                    object obg = sl.Deserialize(sm);
                    sm.Close();
                    return obg;
                }
            }
            catch
            {
                return null;
            }
        }
        #endregion

        public class Binder : SerializationBinder
        {
            public override Type BindToType(string assemblyName, string typeName)
            {
                Assembly ass = Assembly.GetExecutingAssembly();
                return ass.GetType(typeName);
            }
        }

        #region CAD颜色集合
        /// <summary>
        /// CAD颜色集合
        /// </summary>
        public static Color[] CadColors =
        {
            Color.FromRgb(255,0,0),
            Color.FromRgb(255,255,0),
            Color.FromRgb(0,255,0),
            Color.FromRgb(0,255,255),
            Color.FromRgb(0,0,255),
            Color.FromRgb(255,0,255),
            Color.FromRgb(255,255,255),
            Color.FromRgb(128,128,128),
            Color.FromRgb(192,192,192),
            Color.FromRgb(255,0,0),
            Color.FromRgb(255,127,127),
            Color.FromRgb(204,0,0),
            Color.FromRgb(204,102,102),
            Color.FromRgb(153,0,0),
            Color.FromRgb(153,76,76),
            Color.FromRgb(127,0,0),
            Color.FromRgb(127,63,63),
            Color.FromRgb(76,0,0),
            Color.FromRgb(76,38,38),
            Color.FromRgb(255,63,0),
            Color.FromRgb(255,159,127),
            Color.FromRgb(204,51,0),
            Color.FromRgb(204,127,102),
            Color.FromRgb(153,38,0),
            Color.FromRgb(153,95,76),
            Color.FromRgb(127,31,0),
            Color.FromRgb(127,79,63),
            Color.FromRgb(76,19,0),
            Color.FromRgb(76,47,38),
            Color.FromRgb(255,127,0),
            Color.FromRgb(255,191,127),
            Color.FromRgb(204,102,0),
            Color.FromRgb(204,153,102),
            Color.FromRgb(153,76,0),
            Color.FromRgb(153,114,76),
            Color.FromRgb(127,63,0),
            Color.FromRgb(127,95,63),
            Color.FromRgb(76,38,0),
            Color.FromRgb(76,57,38),
            Color.FromRgb(255,191,0),
            Color.FromRgb(255,223,127),
            Color.FromRgb(204,153,0),
            Color.FromRgb(204,178,102),
            Color.FromRgb(153,114,0),
            Color.FromRgb(153,133,76),
            Color.FromRgb(127,95,0),
            Color.FromRgb(127,111,63),
            Color.FromRgb(76,57,0),
            Color.FromRgb(76,66,38),
            Color.FromRgb(255,255,0),
            Color.FromRgb(255,255,127),
            Color.FromRgb(204,204,0),
            Color.FromRgb(204,204,102),
            Color.FromRgb(153,153,0),
            Color.FromRgb(153,153,76),
            Color.FromRgb(127,127,0),
            Color.FromRgb(127,127,63),
            Color.FromRgb(76,76,0),
            Color.FromRgb(76,76,38),
            Color.FromRgb(191,255,0),
            Color.FromRgb(223,255,127),
            Color.FromRgb(153,204,0),
            Color.FromRgb(178,204,102),
            Color.FromRgb(114,153,0),
            Color.FromRgb(133,153,76),
            Color.FromRgb(95,127,0),
            Color.FromRgb(111,127,63),
            Color.FromRgb(57,76,0),
            Color.FromRgb(66,76,38),
            Color.FromRgb(127,255,0),
            Color.FromRgb(191,255,127),
            Color.FromRgb(102,204,0),
            Color.FromRgb(153,204,102),
            Color.FromRgb(76,153,0),
            Color.FromRgb(114,153,76),
            Color.FromRgb(63,127,0),
            Color.FromRgb(95,127,63),
            Color.FromRgb(38,76,0),
            Color.FromRgb(57,76,38),
            Color.FromRgb(63,255,0),
            Color.FromRgb(159,255,127),
            Color.FromRgb(51,204,0),
            Color.FromRgb(127,204,102),
            Color.FromRgb(38,153,0),
            Color.FromRgb(95,153,76),
            Color.FromRgb(31,127,0),
            Color.FromRgb(79,127,63),
            Color.FromRgb(19,76,0),
            Color.FromRgb(47,76,38),
            Color.FromRgb(0,255,0),
            Color.FromRgb(127,255,127),
            Color.FromRgb(0,204,0),
            Color.FromRgb(102,204,102),
            Color.FromRgb(0,153,0),
            Color.FromRgb(76,153,76),
            Color.FromRgb(0,127,0),
            Color.FromRgb(63,127,63),
            Color.FromRgb(0,76,0),
            Color.FromRgb(38,76,38),
            Color.FromRgb(0,255,63),
            Color.FromRgb(127,255,159),
            Color.FromRgb(0,204,51),
            Color.FromRgb(102,204,127),
            Color.FromRgb(0,153,38),
            Color.FromRgb(76,153,95),
            Color.FromRgb(0,127,31),
            Color.FromRgb(63,127,79),
            Color.FromRgb(0,76,19),
            Color.FromRgb(38,76,47),
            Color.FromRgb(0,255,127),
            Color.FromRgb(127,255,191),
            Color.FromRgb(0,204,102),
            Color.FromRgb(102,204,153),
            Color.FromRgb(0,153,76),
            Color.FromRgb(76,153,114),
            Color.FromRgb(0,127,63),
            Color.FromRgb(63,127,95),
            Color.FromRgb(0,76,38),
            Color.FromRgb(38,76,57),
            Color.FromRgb(0,255,191),
            Color.FromRgb(127,255,223),
            Color.FromRgb(0,204,153),
            Color.FromRgb(102,204,178),
            Color.FromRgb(0,153,114),
            Color.FromRgb(76,153,133),
            Color.FromRgb(0,127,95),
            Color.FromRgb(63,127,111),
            Color.FromRgb(0,76,57),
            Color.FromRgb(38,76,66),
            Color.FromRgb(0,255,255),
            Color.FromRgb(127,255,255),
            Color.FromRgb(0,204,204),
            Color.FromRgb(102,204,204),
            Color.FromRgb(0,153,153),
            Color.FromRgb(76,153,153),
            Color.FromRgb(0,127,127),
            Color.FromRgb(63,127,127),
            Color.FromRgb(0,76,76),
            Color.FromRgb(38,76,76),
            Color.FromRgb(0,191,255),
            Color.FromRgb(127,223,255),
            Color.FromRgb(0,153,204),
            Color.FromRgb(102,178,204),
            Color.FromRgb(0,114,153),
            Color.FromRgb(76,133,153),
            Color.FromRgb(0,95,127),
            Color.FromRgb(63,111,127),
            Color.FromRgb(0,57,76),
            Color.FromRgb(38,66,76),
            Color.FromRgb(0,127,255),
            Color.FromRgb(127,191,255),
            Color.FromRgb(0,102,204),
            Color.FromRgb(102,153,204),
            Color.FromRgb(0,76,153),
            Color.FromRgb(76,114,153),
            Color.FromRgb(0,63,127),
            Color.FromRgb(63,95,127),
            Color.FromRgb(0,38,76),
            Color.FromRgb(38,57,76),
            Color.FromRgb(0,63,255),
            Color.FromRgb(127,159,255),
            Color.FromRgb(0,51,204),
            Color.FromRgb(102,127,204),
            Color.FromRgb(0,38,153),
            Color.FromRgb(76,95,153),
            Color.FromRgb(0,31,127),
            Color.FromRgb(63,79,127),
            Color.FromRgb(0,19,76),
            Color.FromRgb(38,47,76),
            Color.FromRgb(0,0,255),
            Color.FromRgb(127,127,255),
            Color.FromRgb(0,0,204),
            Color.FromRgb(102,102,204),
            Color.FromRgb(0,0,153),
            Color.FromRgb(76,76,153),
            Color.FromRgb(0,0,127),
            Color.FromRgb(63,63,127),
            Color.FromRgb(0,0,76),
            Color.FromRgb(38,38,76),
            Color.FromRgb(63,0,255),
            Color.FromRgb(159,127,255),
            Color.FromRgb(51,0,204),
            Color.FromRgb(127,102,204),
            Color.FromRgb(38,0,153),
            Color.FromRgb(95,76,153),
            Color.FromRgb(31,0,127),
            Color.FromRgb(79,63,127),
            Color.FromRgb(19,0,76),
            Color.FromRgb(47,38,76),
            Color.FromRgb(127,0,255),
            Color.FromRgb(191,127,255),
            Color.FromRgb(102,0,204),
            Color.FromRgb(153,102,204),
            Color.FromRgb(76,0,153),
            Color.FromRgb(114,76,153),
            Color.FromRgb(63,0,127),
            Color.FromRgb(95,63,127),
            Color.FromRgb(38,0,76),
            Color.FromRgb(57,38,76),
            Color.FromRgb(191,0,255),
            Color.FromRgb(223,127,255),
            Color.FromRgb(153,0,204),
            Color.FromRgb(178,102,204),
            Color.FromRgb(114,0,153),
            Color.FromRgb(133,76,153),
            Color.FromRgb(95,0,127),
            Color.FromRgb(111,63,127),
            Color.FromRgb(57,0,76),
            Color.FromRgb(66,38,76),
            Color.FromRgb(255,0,255),
            Color.FromRgb(255,127,255),
            Color.FromRgb(204,0,204),
            Color.FromRgb(204,102,204),
            Color.FromRgb(153,0,153),
            Color.FromRgb(153,76,153),
            Color.FromRgb(127,0,127),
            Color.FromRgb(127,63,127),
            Color.FromRgb(76,0,76),
            Color.FromRgb(76,38,76),
            Color.FromRgb(255,0,191),
            Color.FromRgb(255,127,223),
            Color.FromRgb(204,0,153),
            Color.FromRgb(204,102,178),
            Color.FromRgb(153,0,114),
            Color.FromRgb(153,76,133),
            Color.FromRgb(127,0,95),
            Color.FromRgb(127,63,111),
            Color.FromRgb(76,0,57),
            Color.FromRgb(76,38,66),
            Color.FromRgb(255,0,127),
            Color.FromRgb(255,127,191),
            Color.FromRgb(204,0,102),
            Color.FromRgb(204,102,153),
            Color.FromRgb(153,0,76),
            Color.FromRgb(153,76,114),
            Color.FromRgb(127,0,63),
            Color.FromRgb(127,63,95),
            Color.FromRgb(76,0,38),
            Color.FromRgb(76,38,57),
            Color.FromRgb(255,0,63),
            Color.FromRgb(255,127,159),
            Color.FromRgb(204,0,51),
            Color.FromRgb(204,102,127),
            Color.FromRgb(153,0,38),
            Color.FromRgb(153,76,95),
            Color.FromRgb(127,0,31),
            Color.FromRgb(127,63,79),
            Color.FromRgb(76,0,19),
            Color.FromRgb(76,38,47),
            Color.FromRgb(51,51,51),
            Color.FromRgb(91,91,91),
            Color.FromRgb(132,132,132),
            Color.FromRgb(173,173,173),
            Color.FromRgb(214,214,214),
            Color.FromRgb(255,255,255)
        };
        #endregion

        #region 获取更新路径
        /// <summary>
        /// 获取更新路径
        /// </summary>
        /// <param name="settingsPath"></param>
        /// <returns></returns>
        public static string GetUpdatePath(string settingsPath)
        {
            string strUpdatePath = string.Empty;
            try
            {
                XmlDocument xdoc = new XmlDocument();
                xdoc.LoadXml(File.ReadAllText(settingsPath));
                strUpdatePath = xdoc.DocumentElement["Url"].InnerText;
            }
            catch (Exception ex)
            {
                strUpdatePath = ex.uBIMMessage();
            }
            return strUpdatePath;
        }
        #endregion

        #region 获取网络时间
        /// <summary>
        /// 获取网络时间
        /// </summary>
        /// <returns></returns>
        public static DateTime? GetNetTime()
        {
            DateTime? dateTime = null;
            try
            {
                Dictionary<string, string> HeaderList = GetHTTPResponseHeaders("http://www.baidu.com/"); //http://www.ntsc.ac.cn/

                dateTime = DateTime.Parse(HeaderList["Date"]);
            }
            catch
            {
                //忽略
            }
            return dateTime;
        }

        public static Dictionary<string, string> GetHTTPResponseHeaders(string Url)
        {
            Dictionary<string, string> HeaderList = new Dictionary<string, string>();
            WebRequest WebRequestObject = WebRequest.Create(Url);
            //5秒超时
            WebRequestObject.Timeout = 5000;
            WebResponse ResponseObject = WebRequestObject.GetResponse();
            foreach (string HeaderKey in ResponseObject.Headers)
            {
                HeaderList.Add(HeaderKey, ResponseObject.Headers[HeaderKey]);
            }
            ResponseObject.Close();
            return HeaderList;
        }

        public static bool GMTStrParse(string gmtStr, out DateTime gmtTime)
        {
            CultureInfo enUS = new CultureInfo("en-US");
            bool s = DateTime.TryParseExact(gmtStr, "r", enUS, DateTimeStyles.None, out gmtTime);
            return s;
        }
        #endregion

        #region 运行exe
        [DllImport("shell32.dll")]
        public static extern int ShellExecute(IntPtr hwnd, StringBuilder lpszOp, StringBuilder lpszFile, StringBuilder lpszParams, StringBuilder lpszDir, int FsShowCmd);

        /// <summary>
        /// 运行exe
        /// </summary>
        /// <param name="exePath">exe路径</param>
        public static void RunExe(string exePath)
        {
            string exeFileName = Path.GetFileName(exePath);
            string dirPath = Directory.GetParent(exePath).FullName;
            ShellExecute(IntPtr.Zero, new StringBuilder("Open"), new StringBuilder(exeFileName), new StringBuilder(""), new StringBuilder(dirPath), 1);
        }
        #endregion

        #region 安装字体到AutoCad
        /// <summary>
        /// 安装字体到AutoCad
        /// </summary>
        public static void InstallFonts()
        {
            try
            {
                //字体文件路径
                string strInstallPath = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + @"\Autodesk\ApplicationPlugins\uBIM_ReCAD.bundle\Contents\Libs";
                //获取SHX文件路径
                List<string> lstFontFilePath = GetSHXFiles(strInstallPath);

                #region CAD版本对应R版本参数
                Dictionary<string, string> dicCadVersion = new Dictionary<string, string>
                {
                    { "AutoCAD 2002", "R15.0" },
                    { "AutoCAD 2004", "R16.0" },
                    { "AutoCAD 2005", "R16.1" },
                    { "AutoCAD 2006", "R16.2" },
                    { "AutoCAD 2007", "R17.0" },
                    { "AutoCAD 2008", "R17.1" },
                    { "AutoCAD 2009", "R17.2" },
                    { "AutoCAD 2010", "R18.0" },
                    { "AutoCAD 2011", "R18.1" },
                    { "AutoCAD 2012", "R18.2" },
                    { "AutoCAD 2013", "R19.0" },
                    { "AutoCAD 2014", "R19.1" },
                    { "AutoCAD 2015", "R20.0" },
                    { "AutoCAD 2016", "R20.1" },
                    { "AutoCAD 2017", "R21.0" },
                    { "AutoCAD 2018", "R22.0" },
                    { "AutoCAD 2019", "R23.0" },
                    { "AutoCAD 2020", "R24.0" },
                    { "AutoCAD 2021", "R25.0" },
                    { "AutoCAD 2022", "R26.0" },
                    { "AutoCAD 2023", "R27.0" },
                    { "AutoCAD 2024", "R28.0" },
                    { "AutoCAD 2025", "R29.0" },
                };
                #endregion

                foreach (var key in dicCadVersion.Keys)
                {
                    //中文版本
                    string chsFontFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), string.Format(@"Autodesk\{0}\{1}\chs\Support", key, dicCadVersion[key]));
                    if (Directory.Exists(chsFontFolder))
                    {
                        foreach (var fontFilePath in lstFontFilePath)
                        {
                            string strFontName = Path.GetFileName(fontFilePath);
                            string targetFontPath = Path.Combine(chsFontFolder, strFontName);
                            if (!File.Exists(targetFontPath))
                            {
                                File.Copy(fontFilePath, targetFontPath, true);
                            }
                        }
                    }

                    //英文版本
                    string enFontFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), string.Format(@"Autodesk\{0}\{1}\en\Support", key, dicCadVersion[key]));
                    if (Directory.Exists(enFontFolder))
                    {
                        foreach (var fontFilePath in lstFontFilePath)
                        {
                            string strFontName = Path.GetFileName(fontFilePath);
                            string targetFontPath = Path.Combine(enFontFolder, strFontName);
                            if (!File.Exists(targetFontPath))
                            {
                                File.Copy(fontFilePath, targetFontPath, true);
                            }
                        }
                    }
                }
            }
            catch
            {
                throw;
            }
        }
        #endregion

        #region 获取文件夹下所有的SHX字体文件路径
        /// <summary>
        /// 获取文件夹下所有的SHX字体文件路径
        /// </summary>
        /// <param name="targetDir">目标文件夹</param>
        /// <returns></returns>
        private static List<string> GetSHXFiles(string targetDir)
        {
            List<string> result = new List<string>();
            string[] filePaths = Directory.GetFiles(targetDir);
            foreach (var file in filePaths)
            {
                if (file.ToLower().EndsWith(".shx"))
                {
                    result.Add(file);
                }
            }
            return result;
        }
        #endregion
    }
    public static class ExceptionEx
    {
        /// <summary>
        /// 异常信息处理
        /// </summary>
        /// <param name="exception">异常</param>
        /// <returns></returns>
        public static string uBIMMessage(this Exception exception)
        {
            return exception.Message.Replace("WW.CAD", "uBIM.CAD");
        }

        /// <summary>
        /// 异常信息处理
        /// </summary>
        /// <param name="exception">异常</param>
        /// <returns></returns>
        public static string uBIMStackTrace(this Exception exception)
        {
            return exception.uBIMStackTrace().Replace("WW.CAD", "uBIM.CAD");
        }
    }
}
