﻿using System;
using System.Diagnostics;
using System.Reflection;
using System.Windows;
namespace uBIM.Checker.AOP
{
    /// <summary>
    /// 许可检查
    /// </summary>
    public class LicenseChecker
    {
        #region 单例
        //Lazy默认是线程安全的
        private static readonly Lazy<LicenseChecker> instance = new Lazy<LicenseChecker>(() => new LicenseChecker());
        public static LicenseChecker Instance
        {
            get
            {
                return instance.Value;
            }
        }
        #endregion
        public uBIM.PersonLicense.Checker Checker { get; set; }

        private LicenseChecker()
        {
            RunChecker();
        }


        /// <summary>
        /// 启动检查器
        /// </summary>
        public void RunChecker()
        {
            Checker = new uBIM.PersonLicense.Checker(Assembly.GetExecutingAssembly());
            Checker.Run();
            CheckLicenseState("", false);
        }
        /// <summary>
        /// 检查许可状态（是否有效）
        /// </summary>
        /// <param name="showMsg">是否显示提示消息</param>
        /// <returns>是否有效</returns>
        public bool CheckLicenseState(string title = "uBIM", bool showMsg = true)
        {
            bool result = !Checker.OutLimit;
            Trace.TraceInformation(result.ToString());
            if (showMsg)
            {
                string msg;
                if (Checker.LimitDate == DateTime.MinValue)
                {
                    msg = "未激活！\r\n\r\n点击选项卡中的【关于】按钮，在弹出窗的右下角处进行激活！";
                    MessageBox.Show(msg, title, MessageBoxButton.OK, MessageBoxImage.Information);
                }
                else if (DateTime.Now.Date > Checker.LimitDate)
                {
                    msg = string.Format("{0}\r\n有效期至：{1}", "已过期", Checker.LimitDate.ToString("yyyy-MM-dd"));
                    MessageBox.Show(msg, title, MessageBoxButton.OK, MessageBoxImage.Information);
                }
            }
            return result;
        }
        /// <summary>
        /// 注销登录
        /// </summary>
        /// <returns></returns>
        public void Logout()
        {

        }
    }
}
